@@ -1,74 +1,102 @@
 """Postprocessing utils for YOLO3."""
-from typing import Dict, Tuple, List
+from typing import Dict, Tuple, List, Any
 import numpy as np
 import tensorflow as tf
 
 from PIL import Image, ImageDraw, ImageFont
 from iva_applications.yolo2.postprocess import get_spaced_colors
+from iva_applications.mscoco17.config import CLASS_NAMES
 
 
-ANCHORS = [
+YOLO3_ANCHORS = [
         (10, 13), (16, 30), (33, 23),
         (30, 61), (62, 45), (59, 119),
         (116, 90), (156, 198), (373, 326)
 ]
-
+TINY_YOLO3_ANCHORS = [
+        (10, 14), (23, 27), (37, 58),
+        (81, 82), (135, 169), (344, 319),
+]
 
 MAX_OUT_SIZE = 80
 
 
-def draw_boxes(img_names, boxes_dicts, class_names, model_size) -> Image:
+def draw_boxes(
+        img: Image,
+        boxes: Any,
+        font: str = '/usr/share/fonts/liberation/LiberationSans-Regular.ttf') -> Image:
     """
-    Draws detected boxes.
+    Draws detected boxes on a single image.
 
     Parameters
     ----------
-    img_names
-        A list of input images names.
-    boxes_dicts
-        A class-to-boxes dictionary.
-    class_names
-        A class names list.
-    model_size
-        The input size of the model.
-
+    img
+        Pillow image used to be drawn at
+    boxes
+        boxes that should be drawn (in dict or np.ndarray format)
+    font
+        any font
     Returns
     -------
-        Image
+    Pillow image with drawn boxes
     """
-    colors = get_spaced_colors(len(class_names))
-    for _, img_name, boxes_dict in zip(range(len(img_names)), img_names, boxes_dicts):
-        img = Image.open(img_name)
-        draw = ImageDraw.Draw(img)
-        font = ImageFont.truetype(font='/usr/share/fonts/liberation/LiberationSans-Regular.ttf',
-                                  size=(img.size[0] + img.size[1]) // 100)
-        resize_factor = \
-            (img.size[0] / model_size[0], img.size[1] / model_size[1])
-        for cls, _ in enumerate(class_names):
-            boxes = boxes_dict[cls]
-            if np.size(boxes) != 0:
+    draw = ImageDraw.Draw(img)
+    font = ImageFont.truetype(font=font,
+                              size=(img.size[0] + img.size[1]) // 100)
+    colors = get_spaced_colors(len(CLASS_NAMES))
+    if isinstance(boxes, dict):
+        for cls in list(boxes.keys()):
+            box_ = boxes[cls]
+            if np.shape(box_)[0] != 0:
+                box = box_[0]
                 color = colors[cls]
-                for box in boxes:
-                    xy_, confidence = box[:4], box[4]
-                    xy_ = [xy_[i] * resize_factor[i % 2] for i in range(4)]
-                    x_0, y_0 = xy_[0], xy_[1]
-                    thickness = (img.size[0] + img.size[1]) // 200
-                    for tick in np.linspace(0, 1, thickness):
-                        xy_[0], xy_[1] = xy_[0] + tick, xy_[1] + tick
-                        xy_[2], xy_[3] = xy_[2] - tick, xy_[3] - tick
-                        draw.rectangle(xy_, outline=tuple(color))
-                    text = '{} {:.1f}%'.format(class_names[cls],
-                                               confidence * 100)
-                    text_size = draw.textsize(text, font=font)
-                    draw.rectangle(
-                        [x_0, y_0 - text_size[1], x_0 + text_size[0], y_0],
-                        fill=tuple(color))
-                    draw.text((x_0, y_0 - text_size[1]), text, fill='black',
-                              font=font)
-                    print('{} {:.2f}%'.format(class_names[cls],
-                                              confidence * 100))
-        img = img.convert('RGB')
-        return img
+                xy_coords, confidence = box[:4], box[4]
+                xy_coords = np.asarray([xy_coords[0], xy_coords[1], xy_coords[2], xy_coords[3]])
+                x0_coord, y0_coord = xy_coords[0], xy_coords[1]
+                thickness = (img.size[0] + img.size[1]) // 200
+                for tick in np.linspace(0, 1, thickness):
+                    xy_coords[0], xy_coords[1] = xy_coords[0] + tick, xy_coords[1] + tick
+                    xy_coords[2], xy_coords[3] = xy_coords[2] - tick, xy_coords[3] - tick
+                    draw.rectangle(xy_coords, outline=tuple(color))
+                text = '{} {:.1f}%'.format(CLASS_NAMES[cls],
+                                           confidence * 100)
+                text_size = draw.textsize(text, font=font)
+                draw.rectangle(
+                    [x0_coord, y0_coord - text_size[1], x0_coord + text_size[0], y0_coord],
+                    fill=tuple(color))
+                draw.text((x0_coord, y0_coord - text_size[1]), text, fill='black',
+                          font=font)
+    elif isinstance(boxes, np.ndarray):
+        confidence = 0
+        for cls in range(boxes.shape[0]):
+            box = boxes[cls]
+            color = colors[int(box[0])]
+            class_ = int(box[0])
+            if box.shape[0] == 6:
+                xy_coords, confidence = box[1:5], box[5]
+            else:
+                xy_coords = box[1:5]
+            xy_coords = np.asarray([xy_coords[0], xy_coords[1], xy_coords[2], xy_coords[3]])
+            x0_coord, y0_coord = xy_coords[0], xy_coords[1]
+            thickness = (img.size[0] + img.size[1]) // 200
+            for tick in np.linspace(0, 1, thickness):
+                xy_coords[0], xy_coords[1] = xy_coords[0] + tick, xy_coords[1] + tick
+                xy_coords[2], xy_coords[3] = xy_coords[2] - tick, xy_coords[3] - tick
+                draw.rectangle([xy_coords[0], xy_coords[1], xy_coords[2], xy_coords[3]], outline=tuple(color))
+            if box.shape[0] == 6:
+                text = '{} {:.1f}%'.format(CLASS_NAMES[class_], confidence * 100)
+            else:
+                text = '{}'.format(CLASS_NAMES[class_])
+            text_size = draw.textsize(text, font=font)
+            draw.rectangle(
+                [x0_coord, y0_coord - text_size[1], x0_coord + text_size[0], y0_coord],
+                fill=tuple(color))
+            draw.text((x0_coord, y0_coord - text_size[1]), text, fill='white',
+                      font=font)
+    else:
+        raise TypeError('unsupported type of boxes %s' % type(boxes))
+    img = img.convert('RGB')
+    return img
 
 
 def yolo_layer(inputs, n_classes, anchors, img_size):
@@ -201,25 +229,36 @@ def build_detection_graph(
                 anchors: List,
                 iou_threshold: float,
                 confidence_threshold: float):
-    """Build YOLO3 tensorflow postprocessing graph for detection."""
+    """Build Tiny-YOLO3 or YOLO3 tensorflow postprocessing graph for detection."""
     n_classes = len(class_names)
     keys = list(output_tensors.keys())
-    detect0 = yolo_layer(output_tensors[keys[0]],
-                         n_classes=n_classes,
-                         anchors=anchors[6:9],
-                         img_size=input_shape)
-    detect1 = yolo_layer(output_tensors[keys[1]],
-                         n_classes=n_classes,
-                         anchors=anchors[3:6],
-                         img_size=input_shape)
-    detect2 = yolo_layer(output_tensors[keys[2]],
-                         n_classes=n_classes,
-                         anchors=anchors[0:3],
-                         img_size=input_shape)
-    output_tensors = tf.concat([detect0, detect1, detect2], axis=1)
-    output_tensors = build_boxes(output_tensors)
+    if len(keys) == 2:
+        detect0 = yolo_layer(output_tensors[keys[0]],
+                             n_classes=n_classes,
+                             anchors=anchors[3:6],
+                             img_size=input_shape)
+        detect1 = yolo_layer(output_tensors[keys[1]],
+                             n_classes=n_classes,
+                             anchors=anchors[0:3],
+                             img_size=input_shape)
+        output_tensors = tf.concat([detect0, detect1], axis=1)
+    else:
+        detect0 = yolo_layer(output_tensors[keys[0]],
+                             n_classes=n_classes,
+                             anchors=anchors[6:9],
+                             img_size=input_shape)
+        detect1 = yolo_layer(output_tensors[keys[1]],
+                             n_classes=n_classes,
+                             anchors=anchors[3:6],
+                             img_size=input_shape)
+        detect2 = yolo_layer(output_tensors[keys[2]],
+                             n_classes=n_classes,
+                             anchors=anchors[0:3],
+                             img_size=input_shape)
+        output_tensors = tf.concat([detect0, detect1, detect2], axis=1)
+    raw_boxes = build_boxes(output_tensors)
     boxes_dicts = non_max_suppression(
-        output_tensors, n_classes=n_classes,
+        raw_boxes, n_classes=n_classes,
         max_output_size=MAX_OUT_SIZE,
         iou_threshold=iou_threshold,
         confidence_threshold=confidence_threshold)
